@@ -14,6 +14,6 @@ ACTION_PREDICT_EXECUTE_TIME = Histogram('next_action_prediction_execute_time', '
 APP_NAV = Counter(
     'app_nav',
     'Number of nlu prompts and a sum of their corresponding results',
-    ['prompt', 'result']
+    ['prompt', 'result', 'asr_engine']
 
 )
\ No newline at end of file
