@@ -235,7 +235,7 @@ async def load_agent(
 ):
     try:
         if model_server is not None:
-            return await load_from_server(
+            server_model = await load_from_server(
                 Agent(
                     interpreter=interpreter,
                     generator=generator,
@@ -247,9 +247,10 @@ async def load_agent(
                 ),
                 model_server,
             )
+            return [server_model]
 
         elif remote_storage is not None:
-            return Agent.load_from_remote_storage(
+            return [Agent.load_from_remote_storage(
                 remote_storage,
                 model_path,
                 interpreter=interpreter,
@@ -258,12 +259,12 @@ async def load_agent(
                 lock_store=lock_store,
                 action_endpoint=action_endpoint,
                 model_server=model_server,
-            )
+            )]
 
         elif model_path is not None and os.path.exists(model_path):
-            return Agent.load_local_model(
-                model_path,
-                interpreter=interpreter,
+            model_english = Agent.load_local_model(
+                "models/models_english",
+                interpreter=None,
                 generator=generator,
                 tracker_store=tracker_store,
                 lock_store=lock_store,
@@ -272,6 +273,19 @@ async def load_agent(
                 remote_storage=remote_storage,
             )
 
+            model_spanish = Agent.load_local_model(
+                "models/models_spanish",
+                interpreter=None,
+                generator=generator,
+                tracker_store=tracker_store,
+                lock_store=lock_store,
+                action_endpoint=action_endpoint,
+                model_server=model_server,
+                remote_storage=remote_storage,
+            )
+
+            return [model_english, model_spanish]
+
         else:
             warnings.warn("No valid configuration given to load agent.")
             return None
