@@ -93,7 +93,17 @@ def register(
     input_channels: List["InputChannel"], app: Sanic, route: Optional[Text]
 ) -> None:
     async def handler(*args, **kwargs):
-        await app.agent.handle_message(*args, **kwargs)
+        sender_id = args[0].sender_id
+        
+        agent_english = app.agent[0].tracker_store._tracker_store.retrieve(sender_id)
+        agent_spanish = app.agent[1].tracker_store._tracker_store.retrieve(sender_id)
+        lang = 0
+        if agent_english and agent_english.slots and agent_english.slots['session_language']:
+            lang_val = agent_english.slots['session_language'].value
+            if lang_val == 'Spanish':
+                lang = 1
+
+        await app.agent[lang].handle_message(*args, **kwargs)
 
     for channel in input_channels:
         if route:
