@@ -5,15 +5,26 @@ import typing
 from collections import defaultdict, namedtuple
 from typing import Any, Dict, List, Optional, Text, Tuple
 
-from rasa.constants import RESULTS_FILE
+import rasa.utils.io
+from rasa.constants import RESULTS_FILE, PERCENTAGE_KEY
 from rasa.core.utils import pad_lists_to_size
 from rasa.core.events import ActionExecuted, UserUttered
 from rasa.nlu.training_data.formats.markdown import MarkdownWriter
 from rasa.core.trackers import DialogueStateTracker
+from rasa.utils.io import DEFAULT_ENCODING
+import matplotlib
 
 if typing.TYPE_CHECKING:
     from rasa.core.agent import Agent
 
+try:
+    # If the `tkinter` package is available, we can use the `TkAgg` backend
+    import tkinter
+
+    matplotlib.use("TkAgg")
+except ImportError:
+    matplotlib.use("agg")
+
 logger = logging.getLogger(__name__)
 
 StoryEvalution = namedtuple(
@@ -22,7 +33,7 @@ StoryEvalution = namedtuple(
 )
 
 
-class EvaluationStore(object):
+class EvaluationStore:
     """Class storing action, intent and entity predictions and targets."""
 
     def __init__(
@@ -113,9 +124,7 @@ class WronglyPredictedAction(ActionExecuted):
         self, correct_action, predicted_action, policy, confidence, timestamp=None
     ):
         self.predicted_action = predicted_action
-        super(WronglyPredictedAction, self).__init__(
-            correct_action, policy, confidence, timestamp=timestamp
-        )
+        super().__init__(correct_action, policy, confidence, timestamp=timestamp)
 
     def as_story_string(self):
         return "{}   <!-- predicted: {} -->".format(
@@ -130,7 +139,7 @@ class EndToEndUserUtterance(UserUttered):
     `failed_stories.md` output file."""
 
     def as_story_string(self, e2e=True):
-        return super(EndToEndUserUtterance, self).as_story_string(e2e=True)
+        return super().as_story_string(e2e=True)
 
 
 class WronglyClassifiedUserUtterance(UserUttered):
@@ -151,7 +160,7 @@ class WronglyClassifiedUserUtterance(UserUttered):
 
         intent = {"name": eval_store.intent_targets[0]}
 
-        super(WronglyClassifiedUserUtterance, self).__init__(
+        super().__init__(
             event.text,
             intent,
             eval_store.entity_targets,
@@ -167,7 +176,7 @@ class WronglyClassifiedUserUtterance(UserUttered):
         predicted_message = md_format_message(
             self.text, self.predicted_intent, self.predicted_entities
         )
-        return ("{}: {}   <!-- predicted: {}: {} -->").format(
+        return "{}: {}   <!-- predicted: {}: {} -->".format(
             self.intent.get("name"),
             correct_message,
             self.predicted_intent,
@@ -339,7 +348,11 @@ def _predict_tracker_actions(
 
     for event in events[1:]:
         if isinstance(event, ActionExecuted):
-            action_executed_result, policy, confidence = _collect_action_executed_predictions(
+            (
+                action_executed_result,
+                policy,
+                confidence,
+            ) = _collect_action_executed_predictions(
                 processor, partial_tracker, event, fail_on_prediction_errors
             )
             tracker_eval_store.merge_store(action_executed_result)
@@ -393,7 +406,7 @@ def collect_story_predictions(
     correct_dialogues = []
     number_of_stories = len(completed_trackers)
 
-    logger.info("Evaluating {} stories\nProgress:".format(number_of_stories))
+    logger.info(f"Evaluating {number_of_stories} stories\nProgress:")
 
     action_list = []
 
@@ -451,7 +464,7 @@ def log_failed_stories(failed, out_directory):
     if not out_directory:
         return
     with open(
-        os.path.join(out_directory, "failed_stories.md"), "w", encoding="utf-8"
+        os.path.join(out_directory, "failed_stories.md"), "w", encoding=DEFAULT_ENCODING
     ) as f:
         if len(failed) == 0:
             f.write("<!-- All stories passed -->")
@@ -472,10 +485,10 @@ async def test(
     """Run the evaluation of the stories, optionally plot the results."""
     from rasa.nlu.test import get_evaluation_metrics
 
-    completed_trackers = await _generate_trackers(stories, agent, max_stories, e2e)
+    completed_trackers = await _generate_trackers(stories, agent[0], max_stories, e2e)
 
     story_evaluation, _ = collect_story_predictions(
-        completed_trackers, agent, fail_on_prediction_errors, e2e
+        completed_trackers, agent[0], fail_on_prediction_errors, e2e
     )
 
     evaluation_store = story_evaluation.evaluation_store
@@ -524,17 +537,17 @@ def log_evaluation_table(
     include_report=True,
 ):  # pragma: no cover
     """Log the sklearn evaluation metrics."""
-    logger.info("Evaluation Results on {} level:".format(name))
+    logger.info(f"Evaluation Results on {name} level:")
     logger.info(
         "\tCorrect:          {} / {}".format(int(len(golds) * accuracy), len(golds))
     )
-    logger.info("\tF1-Score:         {:.3f}".format(f1))
-    logger.info("\tPrecision:        {:.3f}".format(precision))
-    logger.info("\tAccuracy:         {:.3f}".format(accuracy))
-    logger.info("\tIn-data fraction: {:.3g}".format(in_training_data_fraction))
+    logger.info(f"\tF1-Score:         {f1:.3f}")
+    logger.info(f"\tPrecision:        {precision:.3f}")
+    logger.info(f"\tAccuracy:         {accuracy:.3f}")
+    logger.info(f"\tIn-data fraction: {in_training_data_fraction:.3g}")
 
     if include_report:
-        logger.info("\tClassification report: \n{}".format(report))
+        logger.info(f"\tClassification report: \n{report}")
 
 
 def plot_story_evaluation(
@@ -581,7 +594,6 @@ async def compare_models_in_dir(
     model_dir: Text, stories_file: Text, output: Text
 ) -> None:
     """Evaluates multiple trained models in a directory on a test set."""
-    from rasa.core import utils
     import rasa.utils.io as io_utils
 
     number_correct = defaultdict(list)
@@ -593,23 +605,22 @@ async def compare_models_in_dir(
             if not model.endswith("tar.gz"):
                 continue
 
-            # The model files are named like <policy-name><number>.tar.gz
-            # Remove the number from the name to get the policy name
-            policy_name = "".join(
-                [i for i in os.path.basename(model) if not i.isdigit()]
-            )
+            # The model files are named like <config-name>PERCENTAGE_KEY<number>.tar.gz
+            # Remove the percentage key and number from the name to get the config name
+            config_name = os.path.basename(model).split(PERCENTAGE_KEY)[0]
             number_of_correct_stories = await _evaluate_core_model(model, stories_file)
-            number_correct_in_run[policy_name].append(number_of_correct_stories)
+            number_correct_in_run[config_name].append(number_of_correct_stories)
 
         for k, v in number_correct_in_run.items():
             number_correct[k].append(v)
 
-    utils.dump_obj_as_json_to_file(os.path.join(output, RESULTS_FILE), number_correct)
+    rasa.utils.io.dump_obj_as_json_to_file(
+        os.path.join(output, RESULTS_FILE), number_correct
+    )
 
 
 async def compare_models(models: List[Text], stories_file: Text, output: Text) -> None:
     """Evaluates provided trained models on a test set."""
-    from rasa.core import utils
 
     number_correct = defaultdict(list)
 
@@ -617,13 +628,15 @@ async def compare_models(models: List[Text], stories_file: Text, output: Text) -
         number_of_correct_stories = await _evaluate_core_model(model, stories_file)
         number_correct[os.path.basename(model)].append(number_of_correct_stories)
 
-    utils.dump_obj_as_json_to_file(os.path.join(output, RESULTS_FILE), number_correct)
+    rasa.utils.io.dump_obj_as_json_to_file(
+        os.path.join(output, RESULTS_FILE), number_correct
+    )
 
 
 async def _evaluate_core_model(model: Text, stories_file: Text) -> int:
     from rasa.core.agent import Agent
 
-    logger.info("Evaluating model '{}'".format(model))
+    logger.info(f"Evaluating model '{model}'")
 
     agent = Agent.load(model)
     completed_trackers = await _generate_trackers(stories_file, agent)
@@ -686,7 +699,7 @@ def _plot_curve(
     data = rasa.utils.io.read_json_file(os.path.join(output, RESULTS_FILE))
     x = number_of_examples
 
-    # compute mean of all the runs for keras/embed policies
+    # compute mean of all the runs for different configs
     for label in data.keys():
         if len(data[label]) == 0:
             continue
@@ -707,7 +720,7 @@ def _plot_curve(
 
     plt.savefig(graph_path, format="pdf")
 
-    logger.info("Comparison graph saved to '{}'.".format(graph_path))
+    logger.info(f"Comparison graph saved to '{graph_path}'.")
 
 
 if __name__ == "__main__":
