@@ -155,7 +155,7 @@ class TrackerStore:
             self, sender_id: Text, max_event_history: Optional[int] = None
     ) -> "DialogueStateTracker":
         """Returns tracker or creates one if the retrieval returns None"""
-        tracker = self.retrieve(sender_id)
+        tracker = await self.retrieve(sender_id)
         if max_event_history:
             self.max_event_history = max_event_history
         if tracker is None:
@@ -218,7 +218,7 @@ class TrackerStore:
 
     async def stream_events(self, tracker: DialogueStateTracker) -> None:
         """Streams events to a message broker"""
-        offset = self.number_of_existing_events(tracker.sender_id)
+        offset = await self.number_of_existing_events(tracker.sender_id)
         evts = tracker.events
         start = time.perf_counter()
         form_error = False
@@ -274,9 +274,9 @@ class TrackerStore:
         active_form = curr_state.get('active_form')
         return active_form.get('name') == ENV_AUTHENTICATION_STATE
 
-    def number_of_existing_events(self, sender_id: Text) -> int:
+    async def number_of_existing_events(self, sender_id: Text) -> int:
         """Return number of stored events for a given sender id."""
-        old_tracker = self.retrieve(sender_id)
+        old_tracker = await self.retrieve(sender_id)
         return len(old_tracker.events) if old_tracker else 0
 
     def keys(self) -> Iterable[Text]:
@@ -353,7 +353,7 @@ class InMemoryTrackerStore(TrackerStore):
         serialised = InMemoryTrackerStore.serialise_tracker(tracker)
         self.store[tracker.sender_id] = serialised
 
-    def retrieve(self, sender_id: Text) -> Optional[DialogueStateTracker]:
+    async def retrieve(self, sender_id: Text) -> Optional[DialogueStateTracker]:
         """
         Args:
             sender_id: the message owner ID
@@ -368,9 +368,9 @@ class InMemoryTrackerStore(TrackerStore):
             logger.debug(f"Creating a new tracker for id '{sender_id}'.")
             return None
 
-    def keys(self) -> Iterable[Text]:
+    async def keys(self) -> Iterable[Text]:
         """Returns sender_ids of the Tracker Store in memory"""
-        return self.store.keys()
+        return await self.store.keys()
 
 
 class RedisTrackerStore(TrackerStore):
@@ -412,7 +412,7 @@ class RedisTrackerStore(TrackerStore):
         serialised_tracker = self.serialise_tracker(tracker)
         self.red.set(tracker.sender_id, serialised_tracker, ex=timeout)
 
-    def retrieve(self, sender_id):
+    async def retrieve(self, sender_id):
         """
         Args:
             sender_id: the message owner ID
@@ -426,9 +426,9 @@ class RedisTrackerStore(TrackerStore):
         else:
             return None
 
-    def keys(self) -> Iterable[Text]:
+    async def keys(self) -> Iterable[Text]:
         """Returns keys of the Redis Tracker Store"""
-        return self.red.keys()
+        return await self.red.keys()
 
 
 class DynamoTrackerStore(TrackerStore):
@@ -523,8 +523,8 @@ class DynamoTrackerStore(TrackerStore):
 
 
 class RedisClusterTrackerStore(TrackerStore):
-    def keys(self) -> Iterable[Text]:
-        return self.red_cluster.keys()
+    async def keys(self) -> Iterable[Text]:
+        return await self.red_cluster.keys()
 
     def __init__(
             self,
@@ -582,10 +582,10 @@ class RedisClusterTrackerStore(TrackerStore):
             max_event_history=self.max_event_history,
         )
 
-    def recreate_connection(self):
+    async def recreate_connection(self):
         logger.info("Recreating connection to the Redis cluster...")
         try:
-            self.red_cluster = RedisCluster(
+            self.red_cluster = await RedisCluster(
                 startup_nodes=self.startup_nodes,
                 max_connections=int(self.max_connections),
                 retry_on_timeout=False,
@@ -598,24 +598,24 @@ class RedisClusterTrackerStore(TrackerStore):
             logger.warning(f"Failed to recreate connection to the redis cluster. Error: {exp}")
             return False
 
-    def execute_commands(self, command: str, key=None, value=None, ex=None):
+    async def execute_commands(self, command: str, key=None, value=None, ex=None):
         current_retries = 0
         res = None
         while current_retries < self.max_retries:
             try:
                 pipeline = self.red_cluster.pipeline()
                 if command == "SET":
-                    pipeline.set(key, value, ex=ex)
+                    await pipeline.set(key, value, ex=ex)
                 elif command == "GET":
-                    pipeline.get(key)
+                    await pipeline.get(key)
                 elif command == "DELETE":
-                    pipeline.delete(key)
+                    await pipeline.delete(key)
                 elif command == "PING":
-                    pipeline.ping()
+                    await pipeline.ping()
                 else:
                     raise Exception(f"Unrecognized predefined redis execute commands. Given command: {command}")
 
-                res = pipeline.execute()
+                res = await pipeline.execute()
                 if res and isinstance(res, list) and len(res) > 0:
                     res = res[0]
                     if isinstance(res, ResponseError):
@@ -661,9 +661,9 @@ class RedisClusterTrackerStore(TrackerStore):
         duration = time.perf_counter() - start
         OUTGOING_REQUEST_LATENCY_SEC.labels('redis_save', "NA", "NA").observe(duration)
 
-    def retrieve(self, sender_id):
+    async def retrieve(self, sender_id):
         start = time.perf_counter()
-        stored = self.execute_commands("GET", sender_id)
+        stored = await self.execute_commands("GET", sender_id)
         duration = time.perf_counter() - start
         OUTGOING_REQUEST_LATENCY_SEC.labels('redis_get', "NA", "NA").observe(duration)
 
@@ -672,19 +672,19 @@ class RedisClusterTrackerStore(TrackerStore):
         else:
             return None
 
-    def delete(self, sender_id):
-        return self.execute_commands("DELETE", sender_id)
+    async def delete(self, sender_id):
+        return await self.execute_commands("DELETE", sender_id)
 
-    def ping(self):
-        return self.execute_commands("PING")
+    async def ping(self):
+        return await self.execute_commands("PING")
 
-    def custom_ping(self):
+    async def custom_ping(self):
         while True:
             try:
                 pipeline = self.red_cluster.pipeline()
                 rand_int = random.randint(1, 10000)
                 pipeline.set(f"PING_{rand_int}", "", ex=10)
-                res = pipeline.execute()
+                res = await pipeline.execute()
                 if len(res) > 0:
                     res = res[0]
                     if isinstance(res, ResponseError):
