@@ -6,6 +6,7 @@ from typing import Dict, Text, Any, Optional, Iterator, Generator, Type, List
 
 from rasa.core import events  # pytype: disable=pyi-error
 from rasa.core.actions.action import ACTION_LISTEN_NAME  # pytype: disable=pyi-error
+from rasa.core.constants import USER_INTENT_DISCONNECT
 from rasa.core.conversation import Dialogue  # pytype: disable=pyi-error
 from rasa.core.events import (  # pytype: disable=pyi-error
     UserUttered,
@@ -57,12 +58,14 @@ class AnySlotDict(dict):
         return True
 
 
-class DialogueStateTracker(object):
+class DialogueStateTracker:
     """Maintains the state of a conversation.
 
     The field max_event_history will only give you these last events,
     it can be set in the tracker_store"""
 
+    app_events = {'capture_utterance', 'dtmf', 'get_speech', 'hangup'}
+
     @classmethod
     def from_dict(
         cls,
@@ -75,9 +78,13 @@ class DialogueStateTracker(object):
 
         The dump should be an array of dumped events. When restoring
         the tracker, these events will be replayed to recreate the state."""
+        session_data = {}
+        if len(events_as_dict) > 0:
+            session_data["total_steps"] = events_as_dict[-1].get('total_steps', 0)
+            session_data["total_errors"] = events_as_dict[-1].get('total_errors', 0)
 
         evts = events.deserialise_events(events_as_dict)
-        return cls.from_events(sender_id, evts, slots, max_event_history)
+        return cls.from_events(sender_id, evts, slots, max_event_history, session_data)
 
     @classmethod
     def from_events(
@@ -86,13 +93,14 @@ class DialogueStateTracker(object):
         evts: List[Event],
         slots: Optional[List[Slot]] = None,
         max_event_history: Optional[int] = None,
+        session_data: Optional[Dict] = None,
     ):
-        tracker = cls(sender_id, slots, max_event_history)
+        tracker = cls(sender_id, slots, max_event_history, session_data)
         for e in evts:
             tracker.update(e)
         return tracker
 
-    def __init__(self, sender_id, slots, max_event_history=None):
+    def __init__(self, sender_id, slots, max_event_history=None, session_data=None):
         """Initialize the tracker.
 
         A set of events can be stored externally, and we will run through all
@@ -100,6 +108,8 @@ class DialogueStateTracker(object):
         information we captured while processing messages of the dialogue."""
 
         # maximum number of events to store
+        if session_data is None:
+            session_data = {}
         self._max_event_history = max_event_history
         # list of previously seen events
         self.events = self._create_events([])
@@ -125,6 +135,8 @@ class DialogueStateTracker(object):
         self.latest_message = None
         self.latest_bot_utterance = None
         self._reset()
+        self.total_steps = session_data.get('total_steps', 0)
+        self.total_errors = session_data.get('total_errors', 0)
         self.active_form = {}
 
     ###
@@ -165,7 +177,7 @@ class DialogueStateTracker(object):
         """Generate the past states of this tracker based on the history."""
 
         generated_states = domain.states_for_tracker_history(self)
-        return deque((frozenset(s.items()) for s in generated_states))
+        return deque(frozenset(s.items()) for s in generated_states)
 
     def change_form_to(self, form_name: Text) -> None:
         """Activate or deactivate a form"""
@@ -210,7 +222,7 @@ class DialogueStateTracker(object):
         if key in self.slots:
             return self.slots[key].value
         else:
-            logger.info("Tried to access non existent slot '{}'".format(key))
+            logger.info(f"Tried to access non existent slot '{key}'")
             return None
 
     def get_latest_entity_values(self, entity_type: Text) -> Iterator[Text]:
@@ -234,6 +246,10 @@ class DialogueStateTracker(object):
                 return e.input_channel
         return None
 
+    def from_voice_channel(self):
+        """Check if the input channel is a voice channel"""
+        return self.get_latest_input_channel() and "VOICE" in self.get_latest_input_channel()
+
     def is_paused(self) -> bool:
         """State whether the tracker is currently paused."""
         return self._paused
@@ -251,7 +267,7 @@ class DialogueStateTracker(object):
 
     def events_after_latest_restart(self) -> List[Event]:
         """Return a list of events after the most recent restart."""
-        return list(self.events)[self.idx_after_latest_restart() :]
+        return list(self.events)[self.idx_after_latest_restart():]
 
     def init_copy(self) -> "DialogueStateTracker":
         """Creates a new state tracker with the same initial values."""
@@ -262,7 +278,7 @@ class DialogueStateTracker(object):
         )
 
     def generate_all_prior_trackers(
-        self
+        self,
     ) -> Generator["DialogueStateTracker", None, None]:
         """Returns a generator of the previous trackers of this tracker.
 
@@ -291,8 +307,7 @@ class DialogueStateTracker(object):
                     yield tracker
 
                 elif tracker.active_form.get("rejected"):
-                    for tr in ignored_trackers:
-                        yield tr
+                    yield from ignored_trackers
                     ignored_trackers = []
 
                     if not tracker.active_form.get(
@@ -328,8 +343,7 @@ class DialogueStateTracker(object):
         if tracker.active_form.get("name") is None:
             yield tracker
         elif tracker.active_form.get("rejected"):
-            for tr in ignored_trackers:
-                yield tr
+            yield from ignored_trackers
             yield tracker
 
     def applied_events(self) -> List[Event]:
@@ -377,10 +391,11 @@ class DialogueStateTracker(object):
 
         if not isinstance(dialogue, Dialogue):
             raise ValueError(
-                "story {0} is not of type Dialogue. "
+                "story {} is not of type Dialogue. "
                 "Have you deserialized it?".format(dialogue)
             )
-
+        self.total_steps = getattr(dialogue, "total_steps", 0)
+        self.total_errors = getattr(dialogue, "total_errors", 0)
         self._reset()
         self.events.extend(dialogue.events)
         self.replay_events()
@@ -412,13 +427,29 @@ class DialogueStateTracker(object):
         This can be serialised and later used to recover the state
         of this tracker exactly."""
 
-        return Dialogue(self.sender_id, list(self.events))
+        return Dialogue(self.sender_id, list(self.events), self.total_steps, self.total_errors)
+
+    def add_conversation_steps(self, event: Event) -> None:
+        if isinstance(event, BotUttered) and event.data and 'custom' in event.data and 'type' in event.data['custom'] \
+                and event.data['custom']['type'] in self.app_events:
+            self.total_steps += 1
+        elif isinstance(event, UserUttered) and event.text is not None and not event.text.startswith('/greet') \
+                and not event.text.startswith('/restart') \
+                and not (event.text == '' and event.intent
+                         and 'name' in event.intent and event.intent['name'] == "out_of_scope"
+                         and 'confidence' in event.intent and event.intent['confidence'] == 1):
+            self.total_steps += 1
 
     def update(self, event: Event, domain: Optional[Domain] = None) -> None:
         """Modify the state of the tracker according to an ``Event``. """
         if not isinstance(event, Event):  # pragma: no cover
             raise ValueError("event to log must be an instance of a subclass of Event.")
 
+        if isinstance(event, UserUttered) and self.latest_message:
+            if self.conversation_ended() and event.intent.get("name") == USER_INTENT_DISCONNECT:
+                logger.info(f"[{self.sender_id}] - Conversation has been ended. Ignoring duplicate disconnect messages.")
+                return
+
         self.events.append(event)
         event.apply_to(self)
 
@@ -427,7 +458,7 @@ class DialogueStateTracker(object):
             for e in domain.slots_for_entities(event.parse_data["entities"]):
                 self.update(e)
 
-    def export_stories(self, e2e=False) -> Text:
+    def export_stories(self, e2e: bool = False) -> Text:
         """Dump the tracker as a story in the Rasa Core story format.
 
         Returns the dumped tracker as a string."""
@@ -438,9 +469,11 @@ class DialogueStateTracker(object):
 
     def export_stories_to_file(self, export_path: Text = "debug.md") -> None:
         """Dump the tracker as a story to a file."""
+        import rasa.utils.io
 
-        with open(export_path, "a", encoding="utf-8") as f:
-            f.write(self.export_stories() + "\n")
+        rasa.utils.io.write_text_file(
+            self.export_stories() + "\n", export_path, append=True
+        )
 
     def get_last_event_for(
         self,
@@ -550,6 +583,10 @@ class DialogueStateTracker(object):
 
         self.followup_action = None
 
+    def conversation_ended(self):
+        """Check if the latest message is the end call trigger"""
+        return self.latest_message and self.latest_message.intent.get("name") == USER_INTENT_DISCONNECT
+
     def _merge_slots(
         self, entities: Optional[List[Dict[Text, Any]]] = None
     ) -> List[SlotSet]:
