@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 import asyncio
 import json
 import logging
@@ -32,6 +31,10 @@ logger = logging.getLogger(__name__)
 
 
 class EndToEndReader(MarkdownReader):
+    def __init__(self) -> None:
+        super().__init__()
+        self._regex_interpreter = RegexInterpreter()
+
     def _parse_item(self, line: Text) -> Optional["Message"]:
         """Parses an md list item line based on the current section type.
 
@@ -41,25 +44,39 @@ class EndToEndReader(MarkdownReader):
             DOCS_BASE_URL
         )
 
-        item_regex = re.compile(r"\s*(.+?):\s*(.*)")
+        # Match three groups:
+        # 1) Potential "form" annotation
+        # 2) The correct intent
+        # 3) Optional entities
+        # 4) The message text
+        form_group = fr"({FORM_PREFIX}\s*)*"
+        item_regex = re.compile(r"\s*" + form_group + r"([^{}]+?)({.*})*:\s*(.*)")
         match = re.match(item_regex, line)
-        if match:
-            intent = match.group(1)
-            self.current_title = intent
-            message = match.group(2)
-            example = self._parse_training_example(message)
-            example.data["true_intent"] = intent
-            return example
 
-        raise ValueError(
-            "Encountered invalid end-to-end format for message "
-            "`{}`. Please visit the documentation page on "
-            "end-to-end evaluation at {}/user-guide/evaluating-models/"
-            "end-to-end-evaluation/".format(line, DOCS_BASE_URL)
-        )
+        if not match:
+            raise ValueError(
+                "Encountered invalid end-to-end format for message "
+                "`{}`. Please visit the documentation page on "
+                "end-to-end evaluation at {}/user-guide/evaluating-models/"
+                "end-to-end-evaluation/".format(line, DOCS_BASE_URL)
+            )
+
+        intent = match.group(2)
+        self.current_title = intent
+        message = match.group(4)
+        example = self._parse_training_example(message)
+
+        # If the message starts with the `INTENT_MESSAGE_PREFIX` potential entities
+        # are annotated in the json format (e.g. `/greet{"name": "Rasa"})
+        if message.startswith(INTENT_MESSAGE_PREFIX):
+            parsed = self._regex_interpreter.synchronous_parse(message)
+            example.data["entities"] = parsed["entities"]
+
+        example.data["true_intent"] = intent
+        return example
 
 
-class StoryStepBuilder(object):
+class StoryStepBuilder:
     def __init__(self, name):
         self.name = name
         self.story_steps = []
@@ -74,10 +91,10 @@ class StoryStepBuilder(object):
             self.start_checkpoints.append(Checkpoint(name, conditions))
         else:
             if conditions:
-                logger.warning(
+                warnings.warn(
                     "End or intermediate checkpoints "
                     "do not support conditions! "
-                    "(checkpoint: {})".format(name)
+                    f"(checkpoint: {name})"
                 )
             additional_steps = []
             for t in self.current_steps:
@@ -149,7 +166,7 @@ class StoryStepBuilder(object):
         return current_turns
 
 
-class StoryFileReader(object):
+class StoryFileReader:
     """Helper class to read a story file."""
 
     def __init__(
@@ -232,7 +249,7 @@ class StoryFileReader(object):
         """Given a md file reads the contained stories."""
 
         try:
-            with open(filename, "r", encoding="utf-8") as f:
+            with open(filename, "r", encoding=io_utils.DEFAULT_ENCODING) as f:
                 lines = f.readlines()
             reader = StoryFileReader(domain, interpreter, template_variables, use_e2e)
             return await reader.process_lines(lines)
@@ -285,9 +302,7 @@ class StoryFileReader(object):
             parameters = StoryFileReader._parameters_from_json_string(slots_str, line)
             return event_name, parameters
         else:
-            warnings.warn(
-                "Failed to parse action line '{}'. Ignoring this line.".format(line)
-            )
+            warnings.warn(f"Failed to parse action line '{line}'. Ignoring this line.")
             return "", {}
 
     async def process_lines(self, lines: List[Text]) -> List[StoryStep]:
@@ -315,7 +330,7 @@ class StoryFileReader(object):
                     # reached a checkpoint
                     name, conditions = self._parse_event_line(line[1:].strip())
                     self.add_checkpoint(name, conditions)
-                elif re.match(r"^[*\-]\s+{}".format(FORM_PREFIX), line):
+                elif re.match(fr"^[*\-]\s+{FORM_PREFIX}", line):
                     logger.debug(
                         "Skipping line {}, "
                         "because it was generated by "
@@ -335,13 +350,12 @@ class StoryFileReader(object):
                 else:
                     # reached an unknown type of line
                     logger.warning(
-                        "Skipping line {}. "
+                        f"Skipping line {line_num}. "
                         "No valid command found. "
-                        "Line Content: '{}'"
-                        "".format(line_num, line)
+                        f"Line Content: '{line}'"
                     )
             except Exception as e:
-                msg = "Error in line {}: {}".format(line_num, e)
+                msg = f"Error in line {line_num}: {e}"
                 logger.error(msg, exc_info=1)  # pytype: disable=wrong-arg-types
                 raise ValueError(msg)
         self._add_current_stories_to_result()
@@ -398,11 +412,10 @@ class StoryFileReader(object):
         )
         intent_name = utterance.intent.get("name")
         if intent_name not in self.domain.intents:
-            logger.warning(
-                "Found unknown intent '{}' on line {}. "
+            warnings.warn(
+                f"Found unknown intent '{intent_name}' on line {line_num}. "
                 "Please, make sure that all intents are "
                 "listed in your domain yaml."
-                "".format(intent_name, line_num)
             )
         return utterance
 
@@ -417,7 +430,7 @@ class StoryFileReader(object):
         )
         self.current_step_builder.add_user_messages(parsed_messages)
 
-    async def add_e2e_messages(self, e2e_messages, line_num):
+    async def add_e2e_messages(self, e2e_messages: List[Text], line_num: int) -> None:
         if not self.current_step_builder:
             raise StoryParseError(
                 "End-to-end message '{}' at invalid "
