@@ -573,7 +573,7 @@ def create_app(
     async def retrieve_tracker(request: Request, conversation_id: Text):
         """Check if tracker exists in the tracker store with the given conversation_id"""
         try:
-            tracker = app.agent[0].tracker_store.retrieve(conversation_id)
+            tracker = await app.agent[0].tracker_store.retrieve(conversation_id)
             if tracker:
                 return response.json({"sender_id": conversation_id, "exists": True})
             else:
