@@ -1,9 +1,12 @@
+import time
+
 import pytz
 from datetime import datetime
 import copy
 
 import pytest
 from dateutil import parser
+from rasa.core import utils
 from rasa.core.events import (
     Event,
     UserUttered,
@@ -78,6 +81,9 @@ def test_event_has_proper_implementation(one_event, another_event):
     "one_event",
     [
         UserUttered("/greet", {"name": "greet", "confidence": 1.0}, []),
+        UserUttered(metadata={"type": "text"}),
+        UserUttered(metadata=None),
+        UserUttered(text="hi", message_id="1", metadata={"type": "text"}),
         SlotSet("name", "rasa"),
         Restarted(),
         AllSlotsReset(),
@@ -135,6 +141,7 @@ def test_json_parse_user():
             },
             "entities": []
           },
+          "metadata": {},
         }
     # DOCS END
     # fmt: on
@@ -143,6 +150,7 @@ def test_json_parse_user():
         intent={"name": "greet", "confidence": 0.9},
         entities=[],
         parse_data={"intent": {"name": "greet", "confidence": 0.9}, "entities": []},
+        metadata={},
     )
 
 
@@ -227,3 +235,74 @@ def test_json_parse_agent():
     evt = {"event": "agent", "text": "Hey, how are you?"}
     # DOCS END
     assert Event.from_parameters(evt) == AgentUttered("Hey, how are you?")
+
+
+@pytest.mark.parametrize(
+    "event_class",
+    [
+        UserUttered,
+        BotUttered,
+        ActionReverted,
+        Restarted,
+        AllSlotsReset,
+        ConversationResumed,
+        ConversationPaused,
+        StoryExported,
+        UserUtteranceReverted,
+        AgentUttered,
+    ],
+)
+def test_correct_timestamp_setting_for_events_with_no_required_params(event_class):
+    event = event_class()
+    time.sleep(0.01)
+    event2 = event_class()
+
+    assert event.timestamp < event2.timestamp
+
+
+@pytest.mark.parametrize("event_class", [SlotSet, ActionExecuted, FollowupAction])
+def test_correct_timestamp_setting(event_class):
+    event = event_class("test")
+    time.sleep(0.01)
+    event2 = event_class("test")
+
+    assert event.timestamp < event2.timestamp
+
+
+@pytest.mark.parametrize("event_class", utils.all_subclasses(Event))
+def test_event_metadata_dict(event_class):
+    metadata = {"foo": "bar", "quux": 42}
+
+    # Create the event from a dict that will be accepted by the _from_parameters
+    # method of any Event subclass (the values themselves are not important).
+    event = Event.from_parameters(
+        {
+            "metadata": metadata,
+            "event": event_class.type_name,
+            "parse_data": {},
+            "date_time": "2019-11-20T16:09:16Z",
+        }
+    )
+    assert event.as_dict()["metadata"] == metadata
+
+
+@pytest.mark.parametrize("event_class", utils.all_subclasses(Event))
+def test_event_default_metadata(event_class):
+
+    # Create an event without metadata.
+    # When converting the Event to a dict, it should not include a `metadata`
+    # property - unless it's a UserUttered or a BotUttered event (or subclasses
+    # of them), in which case the metadata should be included with a default
+    # value of {}.
+    event = Event.from_parameters(
+        {
+            "event": event_class.type_name,
+            "parse_data": {},
+            "date_time": "2019-11-20T16:09:16Z",
+        }
+    )
+
+    if isinstance(event, BotUttered) or isinstance(event, UserUttered):
+        assert event.as_dict()["metadata"] == {}
+    else:
+        assert "metadata" not in event.as_dict()
