@@ -1,15 +1,27 @@
+import asyncio
 import json
+from collections import deque
+from typing import Text, List
+
 import pytest
 import uuid
+
+from _pytest.monkeypatch import MonkeyPatch
 from aioresponses import aioresponses
+from mock import Mock
 
 import rasa.utils.io
+from rasa.core.channels import UserMessage
+from rasa.core.trackers import DialogueStateTracker
 from rasa.core.events import BotUttered
 from rasa.core.training import interactive
+from rasa.nlu.training_data.loading import RASA, MARKDOWN
 from rasa.utils.endpoints import EndpointConfig
 from rasa.core.actions.action import default_actions
 from rasa.core.domain import Domain
+from rasa.nlu.training_data import Message
 from tests.utilities import latest_request, json_of_latest_request
+from tests.core.conftest import DEFAULT_DOMAIN_PATH_WITH_SLOTS
 
 
 @pytest.fixture
@@ -20,7 +32,7 @@ def mock_endpoint():
 async def test_send_message(mock_endpoint):
     sender_id = uuid.uuid4().hex
 
-    url = "{}/conversations/{}/messages".format(mock_endpoint.url, sender_id)
+    url = f"{mock_endpoint.url}/conversations/{sender_id}/messages"
     with aioresponses() as mocked:
         mocked.post(url, payload={})
 
@@ -38,7 +50,7 @@ async def test_send_message(mock_endpoint):
 async def test_request_prediction(mock_endpoint):
     sender_id = uuid.uuid4().hex
 
-    url = "{}/conversations/{}/predict".format(mock_endpoint.url, sender_id)
+    url = f"{mock_endpoint.url}/conversations/{sender_id}/predict"
 
     with aioresponses() as mocked:
         mocked.post(url, payload={})
@@ -309,7 +321,7 @@ async def test_interactive_domain_persistence(mock_endpoint, tmpdir):
 
     domain_path = tmpdir.join("interactive_domain_save.yml").strpath
 
-    url = "{}/domain".format(mock_endpoint.url)
+    url = f"{mock_endpoint.url}/domain"
     with aioresponses() as mocked:
         mocked.get(url, payload={})
 
@@ -322,3 +334,93 @@ async def test_interactive_domain_persistence(mock_endpoint, tmpdir):
 
     for default_action in default_actions():
         assert default_action.name() not in saved_domain["actions"]
+
+
+async def test_filter_intents_before_save_nlu_file():
+    # Test method interactive._filter_messages
+    from random import choice
+
+    greet = {"intent": "greet", "text_features": [0.5]}
+    goodbye = {"intent": "goodbye", "text_features": [0.5]}
+    test_msgs = [Message("How are you?", greet), Message("I am inevitable", goodbye)]
+
+    domain_file = DEFAULT_DOMAIN_PATH_WITH_SLOTS
+    domain = Domain.load(domain_file)
+    intents = domain.intents
+
+    msgs = test_msgs.copy()
+    if intents:
+        msgs.append(Message("/" + choice(intents), greet))
+
+    assert test_msgs == interactive._filter_messages(msgs)
+
+
+@pytest.mark.parametrize(
+    "path, expected_format",
+    [("bla.json", RASA), ("other.md", MARKDOWN), ("unknown", MARKDOWN)],
+)
+def test_get_nlu_target_format(path: Text, expected_format: Text):
+    assert interactive._get_nlu_target_format(path) == expected_format
+
+
+@pytest.mark.parametrize(
+    "trackers, expected_trackers",
+    [
+        (
+            [DialogueStateTracker.from_events("one", [])],
+            [deque([]), UserMessage.DEFAULT_SENDER_ID],
+        ),
+        (
+            [
+                str(i)
+                for i in range(
+                    interactive.MAX_NUMBER_OF_TRAINING_STORIES_FOR_VISUALIZATION + 1
+                )
+            ],
+            [UserMessage.DEFAULT_SENDER_ID],
+        ),
+    ],
+)
+async def test_initial_plotting_call(
+    mock_endpoint: EndpointConfig,
+    monkeypatch: MonkeyPatch,
+    trackers: List[Text],
+    expected_trackers: List[Text],
+):
+    get_training_trackers = Mock(return_value=trackers)
+    monkeypatch.setattr(
+        interactive, "_get_training_trackers", asyncio.coroutine(get_training_trackers)
+    )
+
+    monkeypatch.setattr(interactive.utils, "is_limit_reached", lambda _, __: True)
+
+    plot_trackers = Mock()
+    monkeypatch.setattr(interactive, "_plot_trackers", asyncio.coroutine(plot_trackers))
+
+    url = f"{mock_endpoint.url}/domain"
+    with aioresponses() as mocked:
+        mocked.get(url, payload={})
+
+        await interactive.record_messages(mock_endpoint)
+
+    get_training_trackers.assert_called_once()
+    plot_trackers.assert_called_once_with(
+        expected_trackers, interactive.DEFAULT_STORY_GRAPH_FILE, mock_endpoint
+    )
+
+
+async def test_not_getting_trackers_when_skipping_visualization(
+    mock_endpoint: EndpointConfig, monkeypatch: MonkeyPatch
+):
+    get_trackers = Mock()
+    monkeypatch.setattr(interactive, "_get_tracker_events_to_plot", get_trackers)
+
+    monkeypatch.setattr(interactive.utils, "is_limit_reached", lambda _, __: True)
+
+    url = f"{mock_endpoint.url}/domain"
+    with aioresponses() as mocked:
+        mocked.get(url, payload={})
+
+        await interactive.record_messages(mock_endpoint, skip_visualization=True)
+
+    get_trackers.assert_not_called()
