@@ -5,6 +5,7 @@ import numpy as np
 import pytest
 
 import rasa.utils.io
+from rasa.utils import train_utils
 from rasa.core import training
 from rasa.core.actions.action import (
     ACTION_DEFAULT_ASK_AFFIRMATION_NAME,
@@ -17,11 +18,13 @@ from rasa.core.actions.action import (
 )
 from rasa.core.constants import USER_INTENT_RESTART, USER_INTENT_BACK
 from rasa.core.channels.channel import UserMessage
-from rasa.core.domain import Domain, InvalidDomain
-from rasa.core.events import ActionExecuted
+from rasa.core.domain import Domain
+from rasa.core.events import ActionExecuted, ConversationPaused
 from rasa.core.featurizers import (
     BinarySingleStateFeaturizer,
+    LabelTokenizerSingleStateFeaturizer,
     MaxHistoryTrackerFeaturizer,
+    FullDialogueTrackerFeaturizer,
 )
 from rasa.core.policies.two_stage_fallback import TwoStageFallbackPolicy
 from rasa.core.policies.embedding_policy import EmbeddingPolicy
@@ -75,15 +78,6 @@ async def train_trackers(domain, augmentation_factor=20):
     )
 
 
-@pytest.fixture(scope="module")
-def loop():
-    loop = asyncio.new_event_loop()
-    asyncio.set_event_loop(loop)
-    loop = rasa.utils.io.enable_async_loop_debugging(loop)
-    yield loop
-    loop.close()
-
-
 # We are going to use class style testing here since unfortunately pytest
 # doesn't support using fixtures as arguments to its own parameterize yet
 # (hence, we can't train a policy, declare it as a fixture and use the
@@ -91,7 +85,7 @@ def loop():
 # Therefore, we are going to reverse this and train the policy within a class
 # and collect the tests in a base class.
 # noinspection PyMethodMayBeStatic
-class PolicyTestCollection(object):
+class PolicyTestCollection:
     """Tests every policy needs to fulfill.
 
     Each policy can declare further tests on its own."""
@@ -120,6 +114,26 @@ class PolicyTestCollection(object):
         policy.train(training_trackers, default_domain)
         return policy
 
+    def test_featurizer(self, trained_policy, tmpdir):
+        assert isinstance(trained_policy.featurizer, MaxHistoryTrackerFeaturizer)
+        assert trained_policy.featurizer.max_history == self.max_history
+        assert isinstance(
+            trained_policy.featurizer.state_featurizer, BinarySingleStateFeaturizer
+        )
+        trained_policy.persist(tmpdir.strpath)
+        loaded = trained_policy.__class__.load(tmpdir.strpath)
+        assert isinstance(loaded.featurizer, MaxHistoryTrackerFeaturizer)
+        assert loaded.featurizer.max_history == self.max_history
+        assert isinstance(
+            loaded.featurizer.state_featurizer, BinarySingleStateFeaturizer
+        )
+
+    async def test_continue_training(self, trained_policy, default_domain):
+        training_trackers = await train_trackers(default_domain, augmentation_factor=0)
+        trained_policy.continue_training(
+            training_trackers, default_domain, **{"epochs": 1}
+        )
+
     async def test_persist_and_load(self, trained_policy, default_domain, tmpdir):
         trained_policy.persist(tmpdir.strpath)
         loaded = trained_policy.__class__.load(tmpdir.strpath)
@@ -156,16 +170,18 @@ class PolicyTestCollection(object):
 
     def test_tf_config(self, trained_policy, tmpdir):
         if hasattr(trained_policy, "session"):
+            import tensorflow as tf
+
             # noinspection PyProtectedMember
-            assert trained_policy.session._config is None
+            assert trained_policy.session._config == tf.Session()._config
             trained_policy.persist(tmpdir.strpath)
             loaded = trained_policy.__class__.load(tmpdir.strpath)
             # noinspection PyProtectedMember
-            assert loaded.session._config is None
+            assert loaded.session._config == tf.Session()._config
 
     @staticmethod
-    def _get_next_action(policy, events, domain):
-        tracker = get_tracker(events)
+    async def _get_next_action(policy, events, domain):
+        tracker = await get_tracker(events)
 
         scores = policy.predict_action_probabilities(tracker, domain)
         index = scores.index(max(scores))
@@ -192,154 +208,6 @@ class TestKerasPolicyWithTfConfig(PolicyTestCollection):
         assert loaded.session._config == session_config()
 
 
-class TestFallbackPolicy(PolicyTestCollection):
-    def create_policy(self, featurizer, priority):
-        p = FallbackPolicy(priority=priority)
-        return p
-
-    @pytest.mark.parametrize(
-        "nlu_confidence, last_action_name, should_nlu_fallback",
-        [
-            (0.1, "some_action", False),
-            (0.1, "action_listen", True),
-            (0.9, "some_action", False),
-            (0.9, "action_listen", False),
-        ],
-    )
-    def test_should_nlu_fallback(
-        self, trained_policy, nlu_confidence, last_action_name, should_nlu_fallback
-    ):
-        assert (
-            trained_policy.should_nlu_fallback(nlu_confidence, last_action_name)
-            is should_nlu_fallback
-        )
-
-
-class TestMappingPolicy(PolicyTestCollection):
-    def create_policy(self, featurizer, priority):
-        p = MappingPolicy()
-        return p
-
-    @pytest.fixture(scope="module")
-    def domain_with_mapping(self):
-        return Domain.load(DEFAULT_DOMAIN_PATH_WITH_MAPPING)
-
-    @pytest.fixture
-    def tracker(self, domain_with_mapping):
-        return DialogueStateTracker(
-            UserMessage.DEFAULT_SENDER_ID, domain_with_mapping.slots
-        )
-
-    @pytest.fixture(
-        params=[
-            ("default", "utter_default"),
-            ("greet", "utter_greet"),
-            (USER_INTENT_RESTART, ACTION_RESTART_NAME),
-            (USER_INTENT_BACK, ACTION_BACK_NAME),
-        ]
-    )
-    def intent_mapping(self, request):
-        return request.param
-
-    def test_predict_mapped_action(self, priority, domain_with_mapping, intent_mapping):
-        policy = self.create_policy(None, priority)
-        events = [
-            ActionExecuted(ACTION_LISTEN_NAME),
-            user_uttered(intent_mapping[0], 1),
-        ]
-
-        assert (
-            self._get_next_action(policy, events, domain_with_mapping)
-            == intent_mapping[1]
-        )
-
-    def test_predict_action_listen(self, priority, domain_with_mapping, intent_mapping):
-        policy = self.create_policy(None, priority)
-        events = [
-            ActionExecuted(ACTION_LISTEN_NAME),
-            user_uttered(intent_mapping[0], 1),
-            ActionExecuted(intent_mapping[1], policy="policy_0_MappingPolicy"),
-        ]
-        tracker = get_tracker(events)
-        scores = policy.predict_action_probabilities(tracker, domain_with_mapping)
-        index = scores.index(max(scores))
-        action_planned = domain_with_mapping.action_names[index]
-        assert action_planned == ACTION_LISTEN_NAME
-        assert scores != [0] * domain_with_mapping.num_actions
-
-    def test_do_not_follow_other_policy(
-        self, priority, domain_with_mapping, intent_mapping
-    ):
-        policy = self.create_policy(None, priority)
-        events = [
-            ActionExecuted(ACTION_LISTEN_NAME),
-            user_uttered(intent_mapping[0], 1),
-            ActionExecuted(intent_mapping[1], policy="other_policy"),
-        ]
-        tracker = get_tracker(events)
-        scores = policy.predict_action_probabilities(tracker, domain_with_mapping)
-        assert scores == [0] * domain_with_mapping.num_actions
-
-
-class TestMemoizationPolicy(PolicyTestCollection):
-    def create_policy(self, featurizer, priority):
-        max_history = None
-        if isinstance(featurizer, MaxHistoryTrackerFeaturizer):
-            max_history = featurizer.max_history
-        p = MemoizationPolicy(priority=priority, max_history=max_history)
-        return p
-
-    async def test_memorise(self, trained_policy, default_domain):
-        trackers = await train_trackers(default_domain, augmentation_factor=20)
-        trained_policy.train(trackers, default_domain)
-        lookup_with_augmentation = trained_policy.lookup
-
-        trackers = [t for t in trackers if not hasattr(t, "is_augmented")]
-
-        all_states, all_actions = trained_policy.featurizer.training_states_and_actions(
-            trackers, default_domain
-        )
-
-        for tracker, states, actions in zip(trackers, all_states, all_actions):
-            recalled = trained_policy.recall(states, tracker, default_domain)
-            assert recalled == default_domain.index_for_action(actions[0])
-
-        nums = np.random.randn(default_domain.num_states)
-        random_states = [{f: num for f, num in zip(default_domain.input_states, nums)}]
-        assert trained_policy._recall_states(random_states) is None
-
-        # compare augmentation for augmentation_factor of 0 and 20:
-        trackers_no_augmentation = await train_trackers(
-            default_domain, augmentation_factor=0
-        )
-        trained_policy.train(trackers_no_augmentation, default_domain)
-        lookup_no_augmentation = trained_policy.lookup
-
-        assert lookup_no_augmentation == lookup_with_augmentation
-
-    def test_memorise_with_nlu(self, trained_policy, default_domain):
-        filename = "data/test_dialogues/default.json"
-        dialogue = read_dialogue_file(filename)
-
-        tracker = DialogueStateTracker(dialogue.name, default_domain.slots)
-        tracker.recreate_from_dialogue(dialogue)
-        states = trained_policy.featurizer.prediction_states([tracker], default_domain)[
-            0
-        ]
-
-        recalled = trained_policy.recall(states, tracker, default_domain)
-        assert recalled is not None
-
-
-class TestAugmentedMemoizationPolicy(PolicyTestCollection):
-    def create_policy(self, featurizer, priority):
-        max_history = None
-        if isinstance(featurizer, MaxHistoryTrackerFeaturizer):
-            max_history = featurizer.max_history
-        p = AugmentedMemoizationPolicy(priority=priority, max_history=max_history)
-        return p
-
-
 class TestSklearnPolicy(PolicyTestCollection):
     def create_policy(self, featurizer, priority, **kwargs):
         p = SklearnPolicy(featurizer, priority, **kwargs)
@@ -463,51 +331,116 @@ class TestSklearnPolicy(PolicyTestCollection):
         policy.train(trackers, domain=default_domain)
 
 
-class TestEmbeddingPolicyNoAttention(PolicyTestCollection):
+class TestEmbeddingPolicy(PolicyTestCollection):
+    def create_policy(self, featurizer, priority):
+        p = EmbeddingPolicy(featurizer=featurizer, priority=priority)
+        return p
+
+    def test_similarity_type(self, trained_policy):
+        assert trained_policy.similarity_type == "inner"
+
+    async def test_gen_batch(self, trained_policy, default_domain):
+        training_trackers = await train_trackers(default_domain, augmentation_factor=0)
+        training_data = trained_policy.featurize_for_training(
+            training_trackers, default_domain
+        )
+        session_data = trained_policy._create_session_data(
+            training_data.X, training_data.y
+        )
+        batch_size = 2
+        batch_x, batch_y = next(
+            train_utils.gen_batch(session_data=session_data, batch_size=batch_size)
+        )
+        assert batch_x.shape[0] == batch_size and batch_y.shape[0] == batch_size
+        assert (
+            batch_x[0].shape == session_data.X[0].shape
+            and batch_y[0].shape == session_data.Y[0].shape
+        )
+        batch_x, batch_y = next(
+            train_utils.gen_batch(
+                session_data=session_data,
+                batch_size=batch_size,
+                batch_strategy="balanced",
+                shuffle=True,
+            )
+        )
+        assert batch_x.shape[0] == batch_size and batch_y.shape[0] == batch_size
+        assert (
+            batch_x[0].shape == session_data.X[0].shape
+            and batch_y[0].shape == session_data.Y[0].shape
+        )
+
+
+class TestEmbeddingPolicyMargin(TestEmbeddingPolicy):
     def create_policy(self, featurizer, priority):
-        # use standard featurizer from EmbeddingPolicy,
-        # since it is using FullDialogueTrackerFeaturizer
         p = EmbeddingPolicy(
-            priority=priority, attn_before_rnn=False, attn_after_rnn=False
+            featurizer=featurizer, priority=priority, **{"loss_type": "margin"}
+        )
+        return p
+
+    def test_similarity_type(self, trained_policy):
+        assert trained_policy.similarity_type == "cosine"
+
+
+class TestEmbeddingPolicyWithEval(TestEmbeddingPolicy):
+    def create_policy(self, featurizer, priority):
+        p = EmbeddingPolicy(
+            featurizer=featurizer,
+            priority=priority,
+            **{"scale_loss": False, "evaluate_on_num_examples": 4},
         )
         return p
 
 
-class TestEmbeddingPolicyAttentionBeforeRNN(PolicyTestCollection):
+class TestEmbeddingPolicyWithFullDialogue(TestEmbeddingPolicy):
     def create_policy(self, featurizer, priority):
         # use standard featurizer from EmbeddingPolicy,
         # since it is using FullDialogueTrackerFeaturizer
-        p = EmbeddingPolicy(
-            priority=priority, attn_before_rnn=True, attn_after_rnn=False
-        )
+        # if max_history is not specified
+        p = EmbeddingPolicy(priority=priority)
         return p
 
+    def test_featurizer(self, trained_policy, tmpdir):
+        assert isinstance(trained_policy.featurizer, FullDialogueTrackerFeaturizer)
+        assert isinstance(
+            trained_policy.featurizer.state_featurizer,
+            LabelTokenizerSingleStateFeaturizer,
+        )
+        trained_policy.persist(tmpdir.strpath)
+        loaded = trained_policy.__class__.load(tmpdir.strpath)
+        assert isinstance(loaded.featurizer, FullDialogueTrackerFeaturizer)
+        assert isinstance(
+            loaded.featurizer.state_featurizer, LabelTokenizerSingleStateFeaturizer
+        )
 
-class TestEmbeddingPolicyAttentionAfterRNN(PolicyTestCollection):
+
+class TestEmbeddingPolicyWithMaxHistory(TestEmbeddingPolicy):
     def create_policy(self, featurizer, priority):
         # use standard featurizer from EmbeddingPolicy,
-        # since it is using FullDialogueTrackerFeaturizer
-        p = EmbeddingPolicy(
-            priority=priority, attn_before_rnn=False, attn_after_rnn=True
-        )
+        # since it is using MaxHistoryTrackerFeaturizer
+        # if max_history is specified
+        p = EmbeddingPolicy(priority=priority, max_history=self.max_history)
         return p
 
-
-class TestEmbeddingPolicyAttentionBoth(PolicyTestCollection):
-    def create_policy(self, featurizer, priority):
-        # use standard featurizer from EmbeddingPolicy,
-        # since it is using FullDialogueTrackerFeaturizer
-        p = EmbeddingPolicy(
-            priority=priority, attn_before_rnn=True, attn_after_rnn=True
+    def test_featurizer(self, trained_policy, tmpdir):
+        assert isinstance(trained_policy.featurizer, MaxHistoryTrackerFeaturizer)
+        assert trained_policy.featurizer.max_history == self.max_history
+        assert isinstance(
+            trained_policy.featurizer.state_featurizer,
+            LabelTokenizerSingleStateFeaturizer,
+        )
+        trained_policy.persist(tmpdir.strpath)
+        loaded = trained_policy.__class__.load(tmpdir.strpath)
+        assert isinstance(loaded.featurizer, MaxHistoryTrackerFeaturizer)
+        assert loaded.featurizer.max_history == self.max_history
+        assert isinstance(
+            loaded.featurizer.state_featurizer, LabelTokenizerSingleStateFeaturizer
         )
-        return p
 
 
-class TestEmbeddingPolicyWithTfConfig(PolicyTestCollection):
+class TestEmbeddingPolicyWithTfConfig(TestEmbeddingPolicy):
     def create_policy(self, featurizer, priority):
-        # use standard featurizer from EmbeddingPolicy,
-        # since it is using FullDialogueTrackerFeaturizer
-        p = EmbeddingPolicy(priority=priority, **tf_defaults())
+        p = EmbeddingPolicy(featurizer=featurizer, priority=priority, **tf_defaults())
         return p
 
     def test_tf_config(self, trained_policy, tmpdir):
@@ -519,12 +452,84 @@ class TestEmbeddingPolicyWithTfConfig(PolicyTestCollection):
         assert loaded.session._config == session_config()
 
 
-class TestFormPolicy(PolicyTestCollection):
+class TestMemoizationPolicy(PolicyTestCollection):
+    def create_policy(self, featurizer, priority):
+        max_history = None
+        if isinstance(featurizer, MaxHistoryTrackerFeaturizer):
+            max_history = featurizer.max_history
+        p = MemoizationPolicy(priority=priority, max_history=max_history)
+        return p
+
+    def test_featurizer(self, trained_policy, tmpdir):
+        assert isinstance(trained_policy.featurizer, MaxHistoryTrackerFeaturizer)
+        assert trained_policy.featurizer.state_featurizer is None
+        trained_policy.persist(tmpdir.strpath)
+        loaded = trained_policy.__class__.load(tmpdir.strpath)
+        assert isinstance(loaded.featurizer, MaxHistoryTrackerFeaturizer)
+        assert loaded.featurizer.state_featurizer is None
+
+    async def test_memorise(self, trained_policy, default_domain):
+        trackers = await train_trackers(default_domain, augmentation_factor=20)
+        trained_policy.train(trackers, default_domain)
+        lookup_with_augmentation = trained_policy.lookup
+
+        trackers = [
+            t for t in trackers if not hasattr(t, "is_augmented") or not t.is_augmented
+        ]
+
+        (
+            all_states,
+            all_actions,
+        ) = trained_policy.featurizer.training_states_and_actions(
+            trackers, default_domain
+        )
+
+        for tracker, states, actions in zip(trackers, all_states, all_actions):
+            recalled = trained_policy.recall(states, tracker, default_domain)
+            assert recalled == default_domain.index_for_action(actions[0])
+
+        nums = np.random.randn(default_domain.num_states)
+        random_states = [{f: num for f, num in zip(default_domain.input_states, nums)}]
+        assert trained_policy._recall_states(random_states) is None
+
+        # compare augmentation for augmentation_factor of 0 and 20:
+        trackers_no_augmentation = await train_trackers(
+            default_domain, augmentation_factor=0
+        )
+        trained_policy.train(trackers_no_augmentation, default_domain)
+        lookup_no_augmentation = trained_policy.lookup
+
+        assert lookup_no_augmentation == lookup_with_augmentation
+
+    def test_memorise_with_nlu(self, trained_policy, default_domain):
+        filename = "data/test_dialogues/default.json"
+        dialogue = read_dialogue_file(filename)
+
+        tracker = DialogueStateTracker(dialogue.name, default_domain.slots)
+        tracker.recreate_from_dialogue(dialogue)
+        states = trained_policy.featurizer.prediction_states([tracker], default_domain)[
+            0
+        ]
+
+        recalled = trained_policy.recall(states, tracker, default_domain)
+        assert recalled is not None
+
+
+class TestAugmentedMemoizationPolicy(TestMemoizationPolicy):
+    def create_policy(self, featurizer, priority):
+        max_history = None
+        if isinstance(featurizer, MaxHistoryTrackerFeaturizer):
+            max_history = featurizer.max_history
+        p = AugmentedMemoizationPolicy(priority=priority, max_history=max_history)
+        return p
+
+
+class TestFormPolicy(TestMemoizationPolicy):
     def create_policy(self, featurizer, priority):
         p = FormPolicy(priority=priority)
         return p
 
-    async def test_memorise(self, trained_policy):
+    async def test_memorise(self, trained_policy, default_domain):
         domain = Domain.load("data/test_domains/form.yml")
         trackers = await training.load_data("data/test_stories/stories_form.md", domain)
         trained_policy.train(trackers, domain)
@@ -564,6 +569,12 @@ class TestFormPolicy(PolicyTestCollection):
                         "prev_utter_ask_continue" in states[0].keys()
                         and "intent_deny" in states[-1].keys()
                     )
+                    # comes from the fact that intent_inform after utter_ask_continue
+                    # is not read from stories
+                    or (
+                        "prev_utter_ask_continue" in states[0].keys()
+                        and "intent_stop" in states[-1].keys()
+                    )
                 )
                 # @formatter:on
             else:
@@ -582,8 +593,135 @@ class TestFormPolicy(PolicyTestCollection):
         random_states = [{f: num for f, num in zip(domain.input_states, nums)}]
         assert trained_policy.recall(random_states, None, domain) is None
 
+    def test_memorise_with_nlu(self, trained_policy, default_domain):
+        pass
 
-class TestTwoStageFallbackPolicy(PolicyTestCollection):
+
+class TestMappingPolicy(PolicyTestCollection):
+    def create_policy(self, featurizer, priority):
+        p = MappingPolicy()
+        return p
+
+    def test_featurizer(self, trained_policy, tmpdir):
+        assert trained_policy.featurizer is None
+        trained_policy.persist(tmpdir.strpath)
+        loaded = trained_policy.__class__.load(tmpdir.strpath)
+        assert loaded.featurizer is None
+
+    @pytest.fixture(scope="module")
+    def domain_with_mapping(self):
+        return Domain.load(DEFAULT_DOMAIN_PATH_WITH_MAPPING)
+
+    @pytest.fixture
+    def tracker(self, domain_with_mapping):
+        return DialogueStateTracker(
+            UserMessage.DEFAULT_SENDER_ID, domain_with_mapping.slots
+        )
+
+    @pytest.fixture(
+        params=[
+            ("default", "utter_default"),
+            ("greet", "utter_greet"),
+            (USER_INTENT_RESTART, ACTION_RESTART_NAME),
+            (USER_INTENT_BACK, ACTION_BACK_NAME),
+        ]
+    )
+    def intent_mapping(self, request):
+        return request.param
+
+    async def test_predict_mapped_action(self, priority, domain_with_mapping, intent_mapping):
+        policy = self.create_policy(None, priority)
+        events = [
+            ActionExecuted(ACTION_LISTEN_NAME),
+            user_uttered(intent_mapping[0], 1),
+        ]
+
+        assert (
+            await self._get_next_action(policy, events, domain_with_mapping)
+            == intent_mapping[1]
+        )
+
+    async def test_restart_if_paused(self, priority, domain_with_mapping):
+        policy = self.create_policy(None, priority)
+        events = [ConversationPaused(), user_uttered(USER_INTENT_RESTART, 1)]
+
+        assert (
+            await self._get_next_action(policy, events, domain_with_mapping)
+            == ACTION_RESTART_NAME
+        )
+
+    async def test_predict_action_listen(self, priority, domain_with_mapping, intent_mapping):
+        policy = self.create_policy(None, priority)
+        events = [
+            ActionExecuted(ACTION_LISTEN_NAME),
+            user_uttered(intent_mapping[0], 1),
+            ActionExecuted(intent_mapping[1], policy="policy_0_MappingPolicy"),
+        ]
+        tracker = await get_tracker(events)
+        scores = policy.predict_action_probabilities(tracker, domain_with_mapping)
+        index = scores.index(max(scores))
+        action_planned = domain_with_mapping.action_names[index]
+        assert action_planned == ACTION_LISTEN_NAME
+        assert scores != [0] * domain_with_mapping.num_actions
+
+    async def test_do_not_follow_other_policy(
+        self, priority, domain_with_mapping, intent_mapping
+    ):
+        policy = self.create_policy(None, priority)
+        events = [
+            ActionExecuted(ACTION_LISTEN_NAME),
+            user_uttered(intent_mapping[0], 1),
+            ActionExecuted(intent_mapping[1], policy="other_policy"),
+        ]
+        tracker = await get_tracker(events)
+        scores = policy.predict_action_probabilities(tracker, domain_with_mapping)
+        assert scores == [0] * domain_with_mapping.num_actions
+
+
+class TestFallbackPolicy(PolicyTestCollection):
+    def create_policy(self, featurizer, priority):
+        p = FallbackPolicy(priority=priority)
+        return p
+
+    def test_featurizer(self, trained_policy, tmpdir):
+        assert trained_policy.featurizer is None
+        trained_policy.persist(tmpdir.strpath)
+        loaded = trained_policy.__class__.load(tmpdir.strpath)
+        assert loaded.featurizer is None
+
+    @pytest.mark.parametrize(
+        "top_confidence, all_confidences, last_action_name, should_nlu_fallback",
+        [
+            (0.1, [0.1], "some_action", False),
+            (0.1, [0.1], "action_listen", True),
+            (0.9, [0.9, 0.1], "some_action", False),
+            (0.9, [0.9, 0.1], "action_listen", False),
+            (0.4, [0.4, 0.35], "some_action", False),
+            (0.4, [0.4, 0.35], "action_listen", True),
+            (0.9, [0.9, 0.85], "action_listen", True),
+        ],
+    )
+    def test_should_nlu_fallback(
+        self,
+        trained_policy,
+        top_confidence,
+        all_confidences,
+        last_action_name,
+        should_nlu_fallback,
+    ):
+        nlu_data = {
+            "intent": {"confidence": top_confidence},
+            "intent_ranking": [
+                {"confidence": confidence} for confidence in all_confidences
+            ],
+        }
+        assert (
+            trained_policy.should_nlu_fallback(nlu_data, last_action_name)
+            is should_nlu_fallback
+        )
+
+
+class TestTwoStageFallbackPolicy(TestFallbackPolicy):
     def create_policy(self, featurizer, priority):
         p = TwoStageFallbackPolicy(
             priority=priority, deny_suggestion_intent_name="deny"
@@ -606,16 +744,16 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
 
     @staticmethod
     async def _get_tracker_after_reverts(events, channel, nlg, domain):
-        tracker = get_tracker(events)
+        tracker = await get_tracker(events)
         action = ActionRevertFallbackEvents()
         events += await action.run(channel, nlg, tracker, domain)
 
-        return get_tracker(events)
+        return await get_tracker(events)
 
-    def test_ask_affirmation(self, trained_policy, default_domain):
+    async def test_ask_affirmation(self, trained_policy, default_domain):
         events = [ActionExecuted(ACTION_LISTEN_NAME), user_uttered("Hi", 0.2)]
 
-        next_action = self._get_next_action(trained_policy, events, default_domain)
+        next_action = await self._get_next_action(trained_policy, events, default_domain)
 
         assert next_action == ACTION_DEFAULT_ASK_AFFIRMATION_NAME
 
@@ -640,7 +778,7 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
             "## sender\n* greet\n    - utter_hello\n* greet\n"
         )
 
-    def test_ask_rephrase(self, trained_policy, default_domain):
+    async def test_ask_rephrase(self, trained_policy, default_domain):
         events = [
             ActionExecuted(ACTION_LISTEN_NAME),
             user_uttered("greet", 0.2),
@@ -649,7 +787,7 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
             user_uttered("deny", 1),
         ]
 
-        next_action = self._get_next_action(trained_policy, events, default_domain)
+        next_action = await self._get_next_action(trained_policy, events, default_domain)
 
         assert next_action == ACTION_DEFAULT_ASK_REPHRASE_NAME
 
@@ -674,7 +812,7 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
         assert "bye" == tracker.latest_message.parse_data["intent"]["name"]
         assert tracker.export_stories() == "## sender\n* bye\n"
 
-    def test_affirm_rephrased_intent(self, trained_policy, default_domain):
+    async def test_affirm_rephrased_intent(self, trained_policy, default_domain):
         events = [
             ActionExecuted(ACTION_LISTEN_NAME),
             user_uttered("greet", 0.2),
@@ -686,7 +824,7 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
             user_uttered("greet", 0.2),
         ]
 
-        next_action = self._get_next_action(trained_policy, events, default_domain)
+        next_action = await self._get_next_action(trained_policy, events, default_domain)
 
         assert next_action == ACTION_DEFAULT_ASK_AFFIRMATION_NAME
 
@@ -714,7 +852,7 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
         assert "bye" == tracker.latest_message.parse_data["intent"]["name"]
         assert tracker.export_stories() == "## sender\n* bye\n"
 
-    def test_denied_rephrasing_affirmation(self, trained_policy, default_domain):
+    async def test_denied_rephrasing_affirmation(self, trained_policy, default_domain):
         events = [
             ActionExecuted(ACTION_LISTEN_NAME),
             user_uttered("greet", 0.2),
@@ -729,7 +867,7 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
             user_uttered("deny", 1),
         ]
 
-        next_action = self._get_next_action(trained_policy, events, default_domain)
+        next_action = await self._get_next_action(trained_policy, events, default_domain)
 
         assert next_action == ACTION_DEFAULT_FALLBACK_NAME
 
@@ -756,7 +894,7 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
             "## sender\n* greet\n    - utter_hello\n* bye\n"
         )
 
-    def test_unknown_instead_affirmation(self, trained_policy, default_domain):
+    async def test_unknown_instead_affirmation(self, trained_policy, default_domain):
         events = [
             ActionExecuted(ACTION_LISTEN_NAME),
             user_uttered("greet", 0.2),
@@ -765,29 +903,13 @@ class TestTwoStageFallbackPolicy(PolicyTestCollection):
             user_uttered("greet", 0.2),
         ]
 
-        next_action = self._get_next_action(trained_policy, events, default_domain)
+        next_action = await self._get_next_action(trained_policy, events, default_domain)
 
         assert next_action == ACTION_DEFAULT_FALLBACK_NAME
 
-    def test_listen_after_hand_off(self, trained_policy, default_domain):
+    async def test_listen_after_hand_off(self, trained_policy, default_domain):
         events = [ActionExecuted(ACTION_DEFAULT_FALLBACK_NAME)]
 
-        next_action = self._get_next_action(trained_policy, events, default_domain)
+        next_action = await self._get_next_action(trained_policy, events, default_domain)
 
         assert next_action == ACTION_LISTEN_NAME
-
-    def test_exception_if_intent_not_present(self, trained_policy):
-        content = """
-                actions:
-                  - utter_hello
-
-                intents:
-                  - greet
-                """
-        domain = Domain.from_yaml(content)
-
-        events = [ActionExecuted(ACTION_DEFAULT_FALLBACK_NAME)]
-
-        tracker = get_tracker(events)
-        with pytest.raises(InvalidDomain):
-            trained_policy.predict_action_probabilities(tracker, domain)
