@@ -69,7 +69,7 @@ async def test_restart_after_retrieval_from_tracker_store(default_domain: Domain
     latest_restart = tr.idx_after_latest_restart()
 
     await store.save(tr)
-    tr2 = store.retrieve("myuser")
+    tr2 = await store.retrieve("myuser")
     latest_restart_after_loading = tr2.idx_after_latest_restart()
     assert latest_restart == latest_restart_after_loading
 
@@ -80,7 +80,7 @@ async def test_tracker_store_remembers_max_history(default_domain: Domain):
     tr.update(Restarted())
 
     store.save(tr)
-    tr2 = store.retrieve("myuser")
+    tr2 = await store.retrieve("myuser")
     assert tr._max_event_history == tr2._max_event_history == 42
 
 
@@ -400,7 +400,7 @@ async def test_stream_events_step_assignment():
         BotUttered("five", data={"custom": {"type": "not_app_event"}}),
         BotUttered("six", data={"custom": {"type": "hangup"}})
     ]
-    tracker = get_tracker(events)
+    tracker = await get_tracker(events)
     await tracker_store.stream_events(tracker)
     while broker._unpublished_messages:
         actual_steps_sequence.insert(0, loads(broker._unpublished_messages.pop())["step_number"])
