@@ -532,34 +532,34 @@ async def test_tracker_dump_e2e_story(default_agent):
     ]
 
 
-def test_get_last_event_for():
+async def test_get_last_event_for():
     events = [ActionExecuted("one"), user_uttered("two", 1)]
 
-    tracker = get_tracker(events)
+    tracker = await get_tracker(events)
 
     assert tracker.get_last_event_for(ActionExecuted).action_name == "one"
 
 
-def test_get_last_event_with_reverted():
+async def test_get_last_event_with_reverted():
     events = [ActionExecuted("one"), ActionReverted(), user_uttered("two", 1)]
 
-    tracker = get_tracker(events)
+    tracker = await get_tracker(events)
 
     assert tracker.get_last_event_for(ActionExecuted) is None
 
 
-def test_get_last_event_for_with_skip():
+async def test_get_last_event_for_with_skip():
     events = [ActionExecuted("one"), user_uttered("two", 1), ActionExecuted("three")]
 
-    tracker = get_tracker(events)
+    tracker = await get_tracker(events)
 
     assert tracker.get_last_event_for(ActionExecuted, skip=1).action_name == "one"
 
 
-def test_get_last_event_for_with_exclude():
+async def test_get_last_event_for_with_exclude():
     events = [ActionExecuted("one"), user_uttered("two", 1), ActionExecuted("three")]
 
-    tracker = get_tracker(events)
+    tracker = await get_tracker(events)
 
     assert (
         tracker.get_last_event_for(
@@ -569,31 +569,31 @@ def test_get_last_event_for_with_exclude():
     )
 
 
-def test_last_executed_has():
+async def test_last_executed_has():
     events = [
         ActionExecuted("one"),
         user_uttered("two", 1),
         ActionExecuted(ACTION_LISTEN_NAME),
     ]
 
-    tracker = get_tracker(events)
+    tracker = await get_tracker(events)
 
     assert tracker.last_executed_action_has("one") is True
 
 
-def test_last_executed_has_not_name():
+async def test_last_executed_has_not_name():
     events = [
         ActionExecuted("one"),
         user_uttered("two", 1),
         ActionExecuted(ACTION_LISTEN_NAME),
     ]
 
-    tracker = get_tracker(events)
+    tracker = await get_tracker(events)
 
     assert tracker.last_executed_action_has("another") is False
 
 
-def test_events_metadata():
+async def test_events_metadata():
     # It should be possible to attach arbitrary metadata to any event and then
     # retrieve it after getting the tracker dict representation.
     events = [
@@ -602,7 +602,8 @@ def test_events_metadata():
         ActionExecuted(ACTION_LISTEN_NAME, metadata={"three": 3}),
     ]
 
-    events = get_tracker(events).current_state(EventVerbosity.ALL)["events"]
+    tracker = await get_tracker(events)
+    events = tracker.current_state(EventVerbosity.ALL)["events"]
     assert events[0]["metadata"] == {"one": 1}
     assert events[1]["metadata"] == {"two": 2}
     assert events[2]["metadata"] == {"three": 3}
